from objects.Arg import Arg
from objects.Return import Return

class Signature:
    """

    Attributes:
        name (str): description
        args (Arg): description
        ret (Return): description
    """
    def __init__(self, raw: str):
        if raw == '':
            self.name= ''
            self.args: Arg = []
            self.ret = Return('')
            return

        self.name = raw.replace('def ', '').split('(')[0]
        args = []
        for a in raw.split('(')[1].split(')')[0].split(','):
            if a != '':
                args.append(Arg(a))
        self.args = args
        self.ret = Return(raw.split(')')[-1])

    def __repr__(self):
        s = f'{self.name}'
        if len(self.args) > 0:
            s += '('
            for a in self.args:
                if a.type is not None:
                    s += f'{a} '
            s += ')'
            if self.ret.type is not None:
                s += f' -> {self.ret}:'
        return s
    
    def vars(self):
        return vars(self)



class Route:
    def __init__(self, raw: str):
        if raw == '':
            self.path = ''
            self.methods = []
            return
        self.path = raw.split('(')[1].split(')')[0].split(',')[0].replace('\'', '')
        self.methods = raw.split('(')[1].split(')')[0].split('[')[1].split(']')[0].replace('\'', '').replace(' ', '').split(',')

    def __repr__(self):
        s = f'{self.path} - {self.methods}'
        return s
    
    def vars(self):
        return vars(self)


class Arg:
    """
    Attributes:
        name (str): description
        type (str): description
    """
    def __init__(self, raw: str):
        spl = raw.split(':')
        if len(spl) < 2:
            self.name = ''
            self.type = ''
        else :
            self.name = raw.split(':')[0].replace(' ', '')
            self.type = raw.split(':')[1].replace(' ', '')

    def __repr__(self):
        s = f'{self.name}[{self.type}]'
        return s
    
    def vars(self):
        return vars(self)

class Arg_Doc:
    """
    Attributes:
        name (str): description
        type (str): description
        arg (Arg): description
    """
    def __init__(self, name: str, type: type):
        self.name = name
        self.type = type
        self.arg = None
        if 'list' in self.type:
            self.arg = Arg_Doc(self.type, self.type.split('[')[1].split(']')[0])

    def __repr__(self):
        s = f'{self.name} - [{self.type}]'
        return s
    
    def vars(self):
        return vars(self)


class Return:
    """
    Attributes:
        type (str): description
    """
    def __init__(self, raw: str):
        spl = raw.split('->')
        if len(spl) < 2:
            self.type = ''
        else :
            self.type = raw.split('->')[-1].replace(' ', '').replace(':', '')

    def __repr__(self):
        s = f'[{self.type}]'
        return s
    
    def vars(self):
        return vars(self)

class Return_Doc:
    """
    Attributes:
        type (str): description
    """
    def __init__(self, type:str):
        self.type = type        
        self.arg = None
        if 'list' in self.type:
            self.arg = Return_Doc(self.type.split('[')[1].split(']')[0])

        
    def __repr__(self):
        s = f'-> {self.type}\t[{self.arg}]'
        return s


from objects.Return import Return_Doc
from objects.Arg import Arg_Doc


class Docstring(): 
    """
    
    Attributes:
        content (str): Human readable string describing the exception.
        args (list[str]): Exception error code.
        att (list[str]): Exception error code.
        ret (list[str]): Exception error code.
    """
    def __init__(self, raw: str, name: str):
        self.content = raw
        args = []
        att = []
        self.name = name
        self.args = []
        self.att = []
        if 'Args:' in raw and 'Returns:' not in raw: # only args, no ret
            self.ret = None
            self.desc  = self.clean_string(raw.split('Args:')[0])
            args = raw.split('Args:')[1].replace('\t', '').replace(' ', '').split('\n')
        elif 'Args:' in raw and 'Returns:' in raw: #both ret and args
            self.ret = raw.split('Returns:')[1].replace('\t', '').split(':')[0].split(' ')
            self.desc  = self.clean_string(raw.split('Args:')[0])
            args = raw.split('Args:')[1].split('Returns:')[0].replace('\t', '').replace(' ', '').split('\n')
        elif 'Args:' not in raw and 'Returns:' in raw: #no args only ret
            args = []
            self.desc  = self.clean_string(raw.split('Returns:')[0])
            self.ret = raw.split('Returns:')[1].replace('\t', '').split(':')[0].split(' ')
        else:
            self.desc  = ''
            args = []
            self.ret = []
        if 'Attributes:' in raw:
            att = raw.split('Attributes:')[1].replace('\t', '').replace(' ', '').split('\n')

        for a in att:
            if a != '':
                nam = (a.split('(')[0], a.split('(')[1].split(')')[0])[0]
                typ = (a.split('(')[0], a.split('(')[1].split(')')[0])[1]
                self.att.append(Arg_Doc(nam, typ))
            
        for a in args:
            if a != '':
                nam = (a.split('(')[0], a.split('(')[1].split(')')[0])[0]
                typ = (a.split('(')[0], a.split('(')[1].split(')')[0])[1]
                self.args.append(Arg_Doc(nam, typ))
        if self.ret is not None:
            for r in self.ret:
                if r != '':
                    self.ret = Return_Doc(r)

    def clean_string(self, string: str) -> str:
        cleaned = string.replace('\n', '')
        for i in range(2, 10):
            if ' ' * i in cleaned:
                cleaned = cleaned.replace(' ' * i, ' ')
        return cleaned[:-2]

    def __repr__(self):
        s = f'\n\tname : {self.name}\n'
        s += f'\targs : {self.args}\n'
        s += f'\tatt : {self.att}\n'
        s += f'\tret : {self.ret}\n'
        return s

    
    def vars(self):
        return vars(self)


from setuptools import setup
setup(name='pygger',
version='0.214',
description='Swagger doc generator for python code',
url='https://github.com/GBernard314/pygger',
author='GBernard314',
author_email='guillaume.bernard31415@gmail.com',
license='MIT',
packages=['objects'],
zip_safe=False)